#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <SDL/SDL.h>
#include <SDL/SDL_rotozoom.h>
#include <math.h>
#include <string.h>
#include <time.h>

#include "configuration.h"
#include "main.h"
#include "square.h"

void square_image_display(int square_rows, int square_cols)
{
	int square_image_name_value=square_value[square_rows][square_cols];
	int square_image_name_length, i;
	double square_image_zoom_width, square_image_zoom_height;

	for((i = square_image_name_value) && (square_image_name_length=0) ; i > 0 ; square_image_name_length++)
		i-=pow(10, square_image_name_length);

	square_image_name_length+=4; //  >>/ . bmp<<
	square_image_name_length+=strlen(SDL_SQUARE_BMP_FOLDER);

	char square_image_name[square_image_name_length];

	sprintf(square_image_name, "%s/%d.bmp", SDL_SQUARE_BMP_FOLDER, square_image_name_value);

	printf("Displaying image %s.\n", square_image_name);

	sdl_square[square_rows][square_cols]=SDL_LoadBMP(square_image_name);

	square_image_zoom_width=(double) SDL_SQUARE_WIDTH / (double)sdl_square[square_rows][square_cols]->w;
	square_image_zoom_height=(double)SDL_SQUARE_HEIGHT / (double)sdl_square[square_rows][square_cols]->h;

	sdl_square[square_rows][square_cols]=zoomSurface(sdl_square[square_rows][square_cols], square_image_zoom_width, square_image_zoom_height, 1);

	SDL_BlitSurface(sdl_square[square_rows][square_cols], NULL, sdl_window, &sdl_square_pos[square_rows][square_cols]);

	SDL_Flip(sdl_window);
}

void square_image_reset(int square_rows, int square_cols, bool square_refresh)
{
	double square_image_zoom_width, square_image_zoom_height;

	sdl_square[square_rows][square_cols]=SDL_LoadBMP(SDL_SQUARE_BMP);

	square_image_zoom_width=(double) SDL_SQUARE_WIDTH / (double)sdl_square[square_rows][square_cols]->w;
	square_image_zoom_height=(double)SDL_SQUARE_HEIGHT / (double)sdl_square[square_rows][square_cols]->h;

	sdl_square[square_rows][square_cols]=zoomSurface(sdl_square[square_rows][square_cols], square_image_zoom_width, square_image_zoom_height, 1);

	SDL_BlitSurface(sdl_square[square_rows][square_cols], NULL, sdl_window, &sdl_square_pos[square_rows][square_cols]);

	if(square_refresh)
		SDL_Flip(sdl_window);
}

void square_value_set(void)
{
	int square_value_rand;
	int square_value_score;
	int i, j;
	int k, l;

	srand(time(NULL));

	for(i=0 ; i < ROWS ; i++)
	{
		for(j=0 ; j < COLS ; j++)
		{
			do
			{
				square_value_score=0;
				square_value_rand=(rand() % (COLS*ROWS/2)) + 1;

				l=j-1;

				for(k=i ; k >= 0 ; (k--) && (l=COLS-1))
				{
					while(l >= 0)
					{
						//printf("--- [%d][%d]\n", k, l);

						if(square_value[k][l] == square_value_rand)
							square_value_score++;

						l--;
					}
				}

				//printf("/// end (%dx%d)///\n", i, j);
			} while(square_value_score >= 2);

			square_value[i][j]=square_value_rand;
		}
	}

	for(i=0 ; i < ROWS ; i++)
	{
		for(j=0 ; j < COLS ; j++)
		{
			printf("Debug : square_value[%d][%d]=%d;\n", i, j, square_value[i][j]);
		}
	}
}

void square_select(int sdl_mouse_x, int sdl_mouse_y)
{
	int i;
	int square_col, square_row;

	if(sdl_mouse_x == 0 || sdl_mouse_y == 0)
		 return;

	for(i=0 ; sdl_mouse_x > 0 ; i++)
	{
		sdl_mouse_x-=SDL_SQUARE_WIDTH+2;
		
		if(sdl_mouse_x == -1 || sdl_mouse_x == 0 || sdl_mouse_x == 1)
			return;
	}

	square_col=i-1;

	for(i=0 ; sdl_mouse_y > 0 ; i++)
	{
		sdl_mouse_y-=SDL_SQUARE_HEIGHT+2;

		if(sdl_mouse_y == -1 || sdl_mouse_y == 0 || sdl_mouse_y == 1)
			 return;
	}

	square_row=i-1;

	if(square_value[square_row][square_col] == 0)
		 return;

	 printf("Debug : square_value[%d][%d]=%d;\n", square_row, square_col, square_value[square_row][square_col]);

	if(!square_selected_1.ispresent)
	{
		square_selected_1.rows=square_row;
		square_selected_1.cols=square_col;
		square_selected_1.value=square_value[square_row][square_col];
		square_selected_1.ispresent=true;

		square_image_display(square_row, square_col);
	}
	else if(!square_selected_2.ispresent)
	{
		square_selected_2.rows=square_row;
		square_selected_2.cols=square_col;
		square_selected_2.value=square_value[square_row][square_col];
		square_selected_2.ispresent=true;

		if(square_selected_1.rows == square_selected_2.rows && square_selected_1.cols == square_selected_2.cols)
		{
			square_selected_2.ispresent=0;
			return;
		}

		square_image_display(square_row, square_col);

		if(square_selected_1.value == square_selected_2.value)
		{
			square_value[square_selected_1.rows][square_selected_1.cols]=0;
			square_value[square_selected_2.rows][square_selected_2.cols]=0;

			square_selected_1.ispresent=0;
			square_selected_2.ispresent=0;

			if(isended())
			{
				SDL_Delay(1000);
				show_end();
			}
		}
		else
		{
			SDL_Delay(1000);

			square_image_reset(square_selected_1.rows, square_selected_1.cols, true);
			square_image_reset(square_selected_2.rows, square_selected_2.cols, true);

			square_selected_1.ispresent=0;
			square_selected_2.ispresent=0;
		}
	}
}
